/******************************************************************
 *
 *    THIS FILE HAS BEEN AUTOGENERATED, DO NOT MODIFY
 *
 ******************************************************************/

#ifndef GEN_{{file_id}}_SERVICES_H_
#define GEN_{{file_id}}_SERVICES_H_

#include "{{dto_header}}"

namespace {{namespace}} {

{% for service in services %}
class {{service.name}} {
 public:
  virtual ~{{service.name}}() = default;
  {% for method in service.methods -%}
  {% if method.has_reply -%}
  virtual void {{method.name}}(::{{namespace}}::{{method.request}} request, ::{{namespace}}::{{method.reply}} reply) = 0;
  {% else -%}
  virtual void {{method.name}}(::{{namespace}}::{{method.request}} request) = 0;
  {% endif -%}
  {% endfor %}
};
{% endfor %}

}

#endif  // GEN_{{file_id}}_SERVICES_H_
